<?php
/*========================================================================
class_PDO.inc.php
------------------------------------------------------------------------
PDO compatible class that currently only supports MySQL databases.
This class is built to provide forward compatibility; porting existing
code to PHP5.1+ and using the PDO functionality will be a breeze...
------------------------------------------------------------------------
needs class:  none
------------------------------------------------------------------------
Version 0.4   07.06.2006
=========================================================================

TODO: 

=========================================================================*/
/**
* @author Jigal van Hemert <jigal@xs4all.nl>
* @version $Revision: 165 $
*/

require_once('base/_class_base.inc.php');

  define('PDO__MYSQL_ATTR_USE_BUFFERED_QUERY', 1);
  define('PDO__MYSQL_ATTR_LOCAL_INFILE', 2);
  define('PDO__MYSQL_ATTR_INIT_COMMAND', 4);
  define('PDO__MYSQL_ATTR_READ_DEFAULT_FILE', 8);
  define('PDO__MYSQL_ATTR_READ_DEFAULT_GROUP', 16);
  define('PDO__MYSQL_ATTR_MAX_BUFFER_SIZE', 32);
  define('PDO__MYSQL_ATTR_DIRECT_QUERY', 64);
    
  define('PDO__ATTR_AUTOCOMMIT', 1);
  define('PDO__ATTR_CASE', 2);
  define('PDO__ATTR_CLIENT_VERSION', 4);
  define('PDO__ATTR_CONNECTION_STATUS', 8);
  define('PDO__ATTR_DRIVER_NAME', 16);
  define('PDO__ATTR_ERRMODE', 32);
  define('PDO__ATTR_FETCH_CATALOG_NAMES', 64);
  define('PDO__ATTR_FETCH_TABLE_NAMES', 128);
  define('PDO__ATTR_ORACLE_NULLS', 256);
  define('PDO__ATTR_PERSISTENT', 512);
  define('PDO__ATTR_PREFETCH', 1024);
  define('PDO__ATTR_SERVER_INFO', 2048);
  define('PDO__ATTR_SERVER_VERSION', 4096);
  define('PDO__ATTR_STATEMENT_CLASS', 8192);
  define('PDO__ATTR_STRINGIFY_FETCHES', 16384);
  define('PDO__ATTR_TIMEOUT', 32768);
  
  define('PDO__CASE_LOWER', 1);
  define('PDO__CASE_NATURAL', 2);
  define('PDO__CASE_UPPPER', 4);

  define('PDO__ERRMODE_SILENT', 1);
  define('PDO__ERRMODE_WARNING', 2);
  define('PDO__ERRMODE_EXCEPTION', 4);
  
  define('PDO__NULL_NATURAL', 1);
  define('PDO__NULL_EMPTY_STRING', 2);
  define('PDO__NULL_TO_STRING', 3);

  define('PDO__FETCH_ORI_NEXT', 1);
  define('PDO__FETCH_ORI_PRIOR', 2);
  define('PDO__FETCH_ORI_FIRST', 4);
  define('PDO__FETCH_ORI_LAST', 8);
  define('PDO__FETCH_ORI_ABS', 16);
  define('PDO__FETCH_ORI_REL', 32);

  define('PDO__CURSOR_FWDONLY', 1);
  define('PDO__CURSOR_SCROLL', 2);

  define('PDO__ERR_NONE', '00000');

  define('PDO__FETCH_COLUMN', 1);
  define('PDO__FETCH_CLASS', 2);
  define('PDO__FETCH_INTO', 4);
  define('PDO__FETCH_NUM', 8);
  define('PDO__FETCH_ASSOC', 16);
  define('PDO__FETCH_BOTH', 32);
  define('PDO__FETCH_CLASSTYPE', 64);
  define('PDO__FETCH_BOUND', 128);
  define('PDO__FETCH_LAZY', 256);
  define('PDO__FETCH_OBJ', 512);
  define('PDO__FETCH_UNIQUE', 1024);
  define('PDO__FETCH_GROUP', 2048);
  
  define('PDO__PARAM_STR', 1);
  define('PDO__PARAM_INPUT_OUTPUT', 2);

class PDO extends base {
  
  /**#@+
   * @internal
   * @var     string
   */
  /**
   * Stores host name or IP address of database server
   */     
  var $host = 'localhost';

  /**
   * Stores port number of MySQL server
   */   	
  var $port = 3306;

  /**
   * Stores username for access to MySQL 
   */   	
  var $user ='';

  /**
   * Stores password for access to MySQL
   */   	
  var $pass='';

  /**
   * Stores name of database to access
   */   	
  var $database='';

  /**
   * Stores most recent query
   */   	
  var $sql;

  /**
   * Stores options
   */
  var $options;
  /**#@-*/

  /**
   * @internal
   * @var    object
   * PDOStatement object
   */      	
  var $pdostatement;

  /**
   * @internal
   * @var    integer
   * Stores number of rows in result of last query
   */         	
  var $num_rows;

  /**
   * @internal
   * @var    array
   * Stores most recently accessed row of data
   */         	
  var $row;
  
  /**
   * Constructor.	
   * @param string $dsn DSN met connectie-informatie: host,port,dbname,unix_soket
   * @param string $user User name for connection
   * @param string $pass Password for connection
   * @param string $options Array with options
   * @return object
   */	
  function PDO() {
    
    //fill options with defaults
    $this->options = array(
      PDO__ATTR_AUTOCOMMIT => 1,
      PDO__ATTR_CASE => PDO__CASE_NATURAL,
      PDO__ATTR_DRIVER_NAME => 'mysql',
      PDO__ATTR_ERRMODE => PDO__ERRMODE_SILENT,
      PDO__ATTR_ORACLE_NULLS => PDO__NULL_NATURAL,
      PDO__ATTR_PERSISTENT => false,
    );
    // determine whether to get configuration from a file or function parameters
    switch (func_num_args()) {
      case 4: //array with driver_options
        $this->options = array_merge($this->options, func_get_arg(3));
      case 3: //password
        $this->pass = func_get_arg(2);
      case 2: //username
        $this->user = func_get_arg(1);
      case 1: //dsn
        $dsn = func_get_arg(0);
        if (preg_match('/^uri:(.*)$/', $dsn, $matches)) {
          preg_match('/^file:\/\/(.*)$/', $matches[1], $matches);
          $dsn = file_get_contents($matches[1]);
          if (strlen($dsn) < 5) {
            $this->error_code = '28000';
            $this->error_info = array('28000', '-1', "Invalid dsn in file or file not accessible");
            return;
          }
        }
        if (!preg_match('/^mysql:(.*)$/', $dsn, $matches)) {
          $this->error_code = '28000';
          $this->error_info = array('28000', '-2', "Only MySQL DSN is currently supported");
          return;
        } else {
          $dsn = $matches[1];
          $dsn = explode(';', $dsn);
          foreach ($dsn as $row) {
            list($key,$val) = explode('=', $row);
            switch ($key) {
              case 'host':
                $this->host = $val;
                break;
              case 'port':
                $this->port = $val;
                break;
              case 'dbname':
                $this->database = $val;
                break;
              case 'unix_socket':
                $this->host = '';
                $this->port = $val;
                break;
              case 'user':
                $this->user = $val;
                break;
              case 'pass':
                $this->pass = $val;
                break;
              default:
                $this->error_code = '28000';
                $this->error_info = array('28000', '-3', "Invalid element ($key) in DSN");
                return;
            }
          }
        }
      case 0: //error!!
        break;
    }
  }
  
  /**
   * destructor (for future PHP 5 use).
   * Closes the connection to the database   
   */
  function __destruct() {
    $this->CloseConnection();
  }
  
  /**
   * Starts a transaction
   * @return bool Success or failure
   */
  function beginTransaction() {
    $this->pdostatement = $this->prepare("START TRANSACTION;");
    $ret = ($this->pdostatement->execute());
    if ($ret === false) {
      $this->error_code = $ret->errorCode();
      $this->error_info = $ret->errorInfo();
    }
    return $ret;
  }
  
  /**
   * Commits series of statements to database
   * @return bool Success or failure
   */
  function commit() {
    if (isset($this->pdostatement)) {
      $this->pdostatement->sql = "COMMIT;";
      $ret = ($this->pdostatement->execute());
      if ($ret === false) {
        $this->error_code = $ret->errorCode();
        $this->error_info = $ret->errorInfo();
      }
      return $ret;
    } else {
      //error commit while not in a series of statements....
      $this->error_code = 'HY000';
      $this->error_info = array('HY000', 1180, "No transaction started or connection broken");
      return false;
    }
  }
  
  /**
   * Execute single statement and return number of affected rows
   * @param string SQL query
   * @return integer Number of affected rows
   */
  function exec($sql) {
    $this->_reset_error();
    if ($this->pdostatement = $this->prepare($sql)) {
      if ($this->pdostatement->execute()) {
        $ret = $this->pdostatement->rowCount();
      } else {
        $ret = false;
      }
    } else {
      $ret = false;
    }
    if ($ret === false) {
      $this->error_code = $this->pdostatement->errorCode();
      $this->error_info = $this->pdostatement->errorInfo();
    }
    return $ret;
  }
  
  /**
   * Retrieve a database connection attribute
   * @param integer Attribute
   * @return mixed|null Attribute
   */
  function getAttribute($attr) {
    switch ($attr) {
      case PDO__ATTR_AUTOCOMMIT :
        $ret = $this->options[PDO__ATTR_AUTOCOMMIT];
        break;
      case PDO__ATTR_CASE :
        $ret = $this->options[PDO__ATTR_CASE];
        break;
      case PDO__ATTR_CLIENT_VERSION :
        $ret = mysql_get_client_info();
        break;
      case PDO__ATTR_CONNECTION_STATUS :
        if (isset($this->pdostatement)) {
          $ret = mysql_ping($this->pdostatement->dbobject);
        } else {
          $ret = false;
        }
        break;
      case PDO__ATTR_DRIVER_NAME :
        $ret = $this->options[PDO__ATTR_DRIVER_NAME];
        break;
      case PDO__ATTR_ERRMODE :
        $ret = $this->options[PDO__ATTR_ERRMODE];
        break;
      case PDO__ATTR_ORACLE_NULLS :
        $ret = $this->options[PDO__ATTR_ORACLE_NULLS];
        break;
      case PDO__ATTR_PERSISTENT :
        $ret = $this->options[PDO__ATTR_PERSISTENT];
        break;
      case PDO__ATTR_PREFETCH :
        $this->error_code = '0A000'; //feature not supported
        $this->error_info = array('0A000', '-4', 'Feature not supported');
        $ret = false;
        break;
      case PDO__ATTR_SERVER_INFO :
        if (isset($this->pdostatement)) {
          $ret = mysql_get_host_info($this->pdostatement->dbobject);
        } else {
          $ret = false;
        }
        break;
      case PDO__ATTR_SERVER_VERSION :
        if (isset($this->pdostatement)) {
          $ret = mysql_get_server_info($this->pdostatement->dbobject);
        } else {
          $ret = false;
        }
        break;
      case PDO__ATTR_TIMEOUT :
        $this->error_code = '0A000'; //feature not supported
        $this->error_info = array('0A000', '-4', 'Feature not supported');
        $ret = false;
        break;
      default:
        $this->error_code = '';
        $this->error_info = array();
        $ret = null;
        break;
    }
    return $ret;
  }
  
  /**
   * Return an array of available PDO drivers
   * @return array List of availabel PDO drivers
   */
  function getAvailableDrivers() {
    return array('mysql');
  }
  
  /**
   * Returns the ID of the last inserted row or sequence value
   * @param string Name of sequence object (ignore in this version)
   * @return string ID of last inserted row
   */
  function lastInsertId ($name='') {
    if (isset($this->pdostatement)) {
      $ret = $this->pdostatement->_lastInsertID();
    } else {
      $ret = false;
    }
    if ($ret === false) {
      $this->error_code = $this->pdostatement->errorCode();
      $this->error_info = $this->pdostatement->errorInfo();
    }
    return $ret;
  }
  
  /**
   * Prepares a statement for execution and returns a statement object
   * @param string SQL statement
   * @param array Driver options
   * @return PDOStatement Object which is to be used for further actions
   */
  function & prepare ($sql, $driver_options=array() ) {
    $this->_reset_error();
    if (!isset($this->pdostatement)) {
      $this->pdostatement = new PDOStatement ($this->host, $this->port, $this->user, $this->pass, $this->database, $this->options);
    } else {
      //todo: set driver_options to pdostatement object
      unset($this->pdostatement->sqlobject);  //reset sql object
    }
    $this->pdostatement->sql = $sql;
    return $this->pdostatement;
  }
  
  /**
   * Excecute SQL statement
   * @param string Statement
   * @param integer Option (optional)
   * @param mixed Various parameters
   * @return PDOStatement|boolean
   */
  function query() {
    $this->_reset_error();
    if (func_num_args() == 1) {
      //simple version, execute query, return PDOStatement
      $this->pdostatement = & $this->prepare (func_get_arg(0), $this->options);
      if ($this->pdostatement->execute()) {
        return $this->pdostatement;
      } else {
        return false;
      }
    } else {
      switch (func_get_arg(1)) {
        case PDO__FETCH_COLUMN:
          if ($this->pdostatement = & $this->prepare(func_get_arg(0), $this->options)) {
            if ($this->pdostatement->execute()) {
              $ret = $this->pdostatement->fetch(PDO__FETCH_NUM);
              if ($ret !== false) {
                $ret = $ret[func_get_arg(2)];
              }
            } else {
              $ret = false;
            }
          } else {
            $ret = false;
          }
          if ($ret === false) {
            $this->error_code = $this->pdostatement->errorCode();
            $this->error_info = $this->pdostatement->errorInfo();
          }
          return $ret;
          break;
        case PDO__FETCH_CLASS :
          if ($this->pdostatement = & $this->prepare(func_get_arg(0), $this->options)) {
            if ($this->pdostatement->execute()) {
              $ret = $this->pdostatement->fetch(PDO__FETCH_CLASS, func_get_arg(2), func_get_arg(3));
            } else {
              $ret = false;
            }
          } else {
            $ret = false;
          }
          if ($ret === false) {
            $this->error_code = $this->pdostatement->errorCode();
            $this->error_info = $this->pdostatement->errorInfo();
          }
          return $ret;
          break;
        case PDO__FETCH_INTO :
          if ($this->pdostatement = & $this->prepare(func_get_arg(0), $this->options)) {
            if ($this->pdostatement->execute()) {
              $ret = $this->pdostatement->fetch(PDO__FETCH_INTO, func_get_arg(2), func_get_arg(3));
            } else {
              $ret = false;
            }
          } else {
            $ret = false;
          }
          if ($ret === false) {
            $this->error_code = $this->pdostatement->errorCode();
            $this->error_info = $this->pdostatement->errorInfo();
          }
          return $ret;
          break;
        default:
          $this->error_code = 'HY0000';
          $this->error_info = array('HY000', -4, "Invalid mode");
          return false;
          break;
      }
    }
  }
  
  
  /**
   * Escapes special characters in value to prevent SQL injections.
   * 
   * Similar to addslashes(), except that this function takes into
   * account the peculiarities of the MySQL client and connection and
   * all non-numerical values are quoted.
   * @param string Value to be escaped.
   * @return string|false Escaped string or false if connection failed
   */                    
  function quote($string, $parameter_type=PDO__PARAM_STR) {
    $ret = false;
    if (!isset($this->pdostatement)) {
      $this->pdostatement = new PDOStatement ($this->host, $this->port, $this->user, $this->pass, $this->database, $this->options);
    }
    if (!$ret = @mysql_real_escape_string($string, $this->pdostatement->dbobject)) {
      if ($this->pdostatement->_GetConnection()) {
        $ret = mysql_real_escape_string($string, $this->pdostatement->dbobject);
      }
    }
    if (is_null($ret)) {
      $ret = false;
    }
    if ($ret == false) { //connection still doesn't work, simulate with addslashes()
      $ret = addslashes($string);
    }
    if ($parameter_type == PDO__PARAM_STR && !is_numeric($ret)) {
        $ret = "'".$ret."'";
    }
    return $ret;
  }
  
  /**
   * Rollback statements since start of transaction
   * @return bool Success or failure
   */
  function rollBack() {
    if (isset($this->pdostatement)) {
      $this->pdostatement->sql = "ROLLBACK;";
      $ret = ($this->pdostatement->execute());
      if ($ret === false) {
        $this->error_code = $ret->errorCode();
        $this->error_info = $ret->errorInfo();
      }
      return $ret;
    } else {
      $this->error_code = 'HY000';
      $this->error_info = array('HY000', 1181, "No transaction started or connection broken");
      return false;
    }
  }
  
  /**
   * Set an attribute
   * @param integer Attribtue
   * @param mixed Value
   * @return bool Success or failure
   */
  function setAttribute ($attr, $value) {
    switch ($attr) {
      
    }
  }
  
}

/**************************************************************************
 *                  CLASS PDOStatement
 *************************************************************************/
class PDOStatement extends eurobase {
  
  /**#@+
   * @internal
   * @var     string
   */
  /**
   * Stores host name or IP address of database server
   */     
  var $host = 'localhost';

  /**
   * Stores port number of MySQL server
   */   	
  var $port=3306;

  /**
   * Stores username for access to MySQL 
   */   	
  var $user;

  /**
   * Stores password for access to MySQL
   */   	
  var $pass;

  /**
   * Stores name of database to access
   */   	
  var $database;

  /**
   * Stores most recent query
   */   	
  var $sql;

  /**
   * Stores options
   */
  var $options;
  /**#@-*/

  /**
   * @internal
   * @var    object
   * Database connection object	 
   */      	
  var $dbobject;

  /**
   * @internal
   * @var    object
   * SQL data object
   */         	
  var $sqlobject;

  /**
   * @internal
   * @var    integer
   * Stores number of rows in result of last query
   */         	
  var $num_rows;

  /**
   * @internal
   * @var    array
   * Stores most recently accessed row of data
   */         	
  var $row;

  /**
   * @internal
   * @var integer
   * Stores mode for fetching data
   */
  var $fetchmode;

  /**
   * @internal
   * @var array
   * Stores information on fetchmode
   */
  var $fetchoptions;

  /**
   * @internal
   * @var array
   * List of input bindings
   */
  var $inputbinding = Array();

  /**
   * @internal
   * @var array
   * List of output bindings
   */
  var $outputbinding;

  /**
   * @internal
   * @var array
   * List of options and values
   */
  var $attributes;
  
  /**
   * Constructor.
   * @param string Hostname
   * @param string Port number
   * @param string User name
   * @param string Password
   * @param string Database name
   * @param array  Various options
   */
  function PDOStatement ($host, $port, $user, $pass, $name, $options) {
    $this->host = $host;
    $this->port = $port;
    $this->user = $user;
    $this->pass = $pass;
    $this->database = $name;
    $this->options = $options;
    $this->fetchmode = PDO__FETCH_BOTH;
  }
  
  /**
   * Bind a column to a variable
   * @param mixed Column to bind to (name or number)
   * @param mixed Variable to bind to
   * @param integer Datatype (optional, not used)
   * @return true
   */
  function bindColumn ($column, &$param, $type=0) {
    if (!isset($param)) $param = '';
    if (is_numeric($column)) $column--;
    $this->outputbinding[$column] = array(&$param, $type);
    return true;
  }
  
  /**
   * Binds parameter in prepared statement to a variable
   * @param mixed Parameter to bind to
   * @param mixed Variable to bind to
   * @param integer Data type (optional)
   * @param integer Length (optional)
   * @param mixed Driver options (optional)
   * @return true
   */
  function bindParam ($param, &$variable, $type=0, $length=0, $options=NULL) {
    if ($type & PDO__PARAM_INPUT_OUTPUT > 0) {
      $this->bindColumn ($param, $variable, ($type & !PDO__PARAM_INPUT_OUTPUT));
    }
    $type = $type & !PDO__PARAM_INPUT_OUTPUT;
    $this->bindValue($param, $variable, $type);
    return true;
  }
  
  /**
   * Binds a numbered or named placeholder to a variable.
   * @param mixed Parameter to bind to
   * @param mixed Variable to bind to parameter
   * @param integer Type of variable
   * @return true
   */
  function bindValue ($param, &$variable, $type=0) {
    $this->inputbinding[$param] = array($variable, $type);
  }

  /**
   * Free resources for the database connection.
   * @return true
   */
  function closeCursor () {
    if (is_resource($this->sqlobject)) mysql_free_result($this->sqlobject);
    $this->row = array();
    return true;
  }
  
  /**
   * Return number of columns in the result set. If no result set is yet present zero will be returned.
   * @return integer Number of columns
   */
  function columnCount () {
    if (is_resource($this->sqlobject)) {
      return mysql_num_fields($this->sqlobject);
    } else {
      return 0;
    }
  }
  
  /**
   * Executes a prepared statement
   * @param array Input parameters for bindParam() (optional)
   * @return boolean False signals failure when executing query
   */
  function execute () {
    // if input parameters were given use bindParam() to process them
    // execute query and report outcome
    global $parcnt;
    $parcnt = 0;
    if (!isset($this->dbobject)) {
      if(!$this->_GetConnection()){
        return false;
      }
    }
    if (!isset($this->sqlobject)) {
      //process input bindings
      if (count($this->inputbinding) > 0) foreach ($this->inputbinding as $key => $value) {
        list($val, $data_type) = $value;
        if (!is_numeric($key)) {
          // replace the marker (not preceeded by a word character or a ':' but followed
          // by a word boundary)
          $this->sql = preg_replace('/(?<![\w:])'.$key.'\b/', PDO::quote($val, $data_type), $this->sql);
        }
      }
      // need to replace ? markers, but a ? may also appear within a string or regexp
      // I assume here that we will only replace ? markers which are not surrounded
      // by non-whitespace
      $this->sql = preg_replace_callback('/(?<!\w)\?(?!\w)/', array($this, '_replace_num_marker'), $this->sql);
      if(!$this->_MysqlQuery()){
        return false;
      }
    }
    return true;
  }
  
  /**
   * Callback function for replacing question markers in prepared statements
   * @internal
   * @param array $matches
   * @return string replacement
   */
  function _replace_num_marker ($matches) {
    global $parcnt;
    $parcnt++;
    if (array_key_exists($parcnt, $this->inputbinding)) {
      list($val, $data_type) = $this->inputbinding[$parcnt];
      return PDO::quote($val, $data_type);
    } else {
      return '';
    }
  }
  
  /**
   * Fetch next row from result set
   * @param integer Fetch style (optional)
   * @param integer Cursor orientation (optional)
   * @param integer Cursor offset (optional)
   * @return mixed Next row in result set
   */
  function fetch ($fetch_style=false, $curs_ori=false, $col_idx=0) {
    //execute query if it hasn't been done yet...
    if (!isset($this->sqlobject)) {
      if (!$this->execute()) {
        return false; 
      }
    }
    //get next row of data
    if ($this->row = @mysql_fetch_array($this->sqlobject, MYSQL_BOTH)) { //try to fetch record
      //process output binding
      if ($fetch_style == false) $fetch_style = $this->fetchmode;
      switch ($fetch_style) {
        case PDO__FETCH_ASSOC :
          foreach(array_keys($this->row) as $key) {
            if (is_numeric($key)) {
              unset($this->row[$key]);
            }
          }
          break;
        case PDO__FETCH_BOTH :
          //done! ;-)
          break;
        case PDO__FETCH_BOUND :
          if (count($this->outputbinding) > 0) foreach ($this->outputbinding as $key => $value) {
            $this->outputbinding[$key][0] = $this->row[$key];
          }
          break;
        case PDO__FETCH_CLASS :
          $ret = new $this->fetchoptions['classname']($this->fetchoptions['ctoargs']);
          foreach(array_keys($this->row) as $key) {
            if (!is_numeric($key)) {
              $ret->$key = $this->row[$key];
            }
          }
          $this->row = & $ret;
          break;
        case PDO__FETCH_INTO :
          if ($curs_ori) {
            $obje = & $curs_ori;
          } else {
            $obje = & $this->fetchoptions['object'];
          }
          foreach(array_keys($this->row) as $key) {
            if (!is_numeric($key)) {
              $obje->$key = $this->row[$key];
            }
          }
          break;
        case PDO__FETCH_LAZY :
          $ret = new stdClass();
          foreach(array_keys($this->row) as $key) {
            $ret->$key = $this->row[$key];
          }
          $this->row = $ret;
          break;
          break;
        case PDO__FETCH_NUM :
          foreach(array_keys($this->row) as $key) {
            if (!is_numeric($key)) {
              unset($this->row[$key]);
            }
          }
          break;
        case PDO__FETCH_OBJ :
          $ret = new stdClass();
          foreach(array_keys($this->row) as $key) {
            if (!is_numeric($key)) {
              $ret->$key = $this->row[$key];
            }
          }
          $this->row = & $ret;
          break;
        default:
          //raise error
          break;
      }
      return $this->row;
    } else { //couldn't fetch a row
      $err = mysql_errno($this->dbobject);
      if ($err != 0) { //real error, otherwise end of data
        $this->error_code = ($this->err2state[$err] != '') ? $this->err2state[$err] : 'HY0000';
        $this->error_info = array($this->error_code, $err, mysql_error($this->dbobject));
      }
      return false;
    }
    /***********************TODO***************************************************
    //check existing query object
    //get next row
    //process bindings
    //return row
    ******************************************************************************/
  }
  
  /**
   * Returns all rows in the result set
   * @param integer Fetch mode, defaults to PDO__FETCH_BOTH
   * @param integer Column index for mode PDO__FETCH_COLUMN
   */
  function fetchAll ($fetch_style=false, $col_idx=0) {
    if ($fetch_style == false) $fetch_style = $this->fetchmode;
    $result = array();
    while ($row = $this->fetch($fetch_style, false, $col_idx) ) {
      $result[] = $row;
    }
    /***********************TODO***************************************************
    // process data according to fetch style
    // especially FETCH_UNIQUE, FETCH_GROUP require extra actions
    ******************************************************************************/
    return $result;
  }
  
  /**
   * Returns value of a single column in the next row
   * @param integer Column index of column to fetch
   * @return mixed|false Column value or fals in case of error
   */
  function fetchColumn ($col_idx=0) {
    return $this->fetch(PDO__FETCH_COLUMN, $col_idx);
  }
  
  /**
   * Returns object with data for next row
   * @param string Class name for object
   * @param array List of arguments for calling the constructor
   */
  function & fetchObject ($class='stdClass', $args='') {
    return $this->fetch(PDO__FETCH_CLASS, $class, $args);
  }
  
  /**
   * Reads attbribute as set by setAttribute
   * @param int Attribute;
   * @return mixed Value of attribute
   */
  function getAttribute ($attr) {
    return $this->attributes[$attr];
  }
  
  /**
   * Returns metadata about a single column in the result set
   * @return false Not implemented
   */
  function getColumnMeta () {
    $this->error_code = '0A000'; //feature not supported
    $this->error_info = array('0A000', '-4', 'Feature not supported');
    return false;
  }

  /**
   * Tries to fetch a new row set from a stored procedure.
   * Not implemented, so returns false. 
   * @return false Not implemented
   */        
  function nextRowset () {
    $this->error_code = '0A000'; //feature not supported
    $this->error_info = array('0A000', '-4', 'Feature not supported');
    return false;
  }

  /**
   * Return the number of rows affected by the most recent query.
   * @return integer|false The number of affected rows or false.
   */      	
  function rowCount () {
    if (is_resource($this->dbobject)) {
      return mysql_affected_rows($this->dbobject);
    } else {
      return false;
    }
  }

  /**
   * Set attribute for future queries
   * @param integer Attribute
   * @param mixed Value of attribute
   */
  function setAttribute ($attr, $value) {
    $this->attributes[$attr] = $value;
    return true;
  }
  
  /**
   * Set the mode for fetching data rows
   * @param integer Mode
   * @param mixed Class name/Column number/Object
   * @param array Constructor arguments in case of mode FETCH_CLASS
   */
  function setFetchMode ($mode, &$class, $args='') {
    switch ($mode) {
      case PDO__FETCH_COLUMN :
        $this->fetchmode = $mode;
        $this->fetchoptions['column'] = $class.""; //make sure copy of data is used and not reference
        break;
      case PDO__FETCH_CLASS :
        $this->fetchmode = $mode;
        $this->fetchoptions['classname'] = $class.""; //make sure copy of data is used and not reference
        $this->fetchoptions['ctoargs'] = $args;
        break;
      case PDO__FETCH_INTO :
        $this->fetchmode = $mode;
        $this->fetchoptions['object'] = $class;
        break;
      default:
        $this->fetchmode = $mode;
        break;
    }
    return true;
  }

  /**
   * Internal function to check the status of the connection to the MySQL server
   * and try to connect if necessary.
   * Sets error code if necessary.        
   * @internal
   * @return bool True if successfuly connected
   */        	
  function _GetConnection(){
    //private method to check and if needed activate the mysql connection
    if (isset($this->dbobject)) {
      return true;
    }
    else{
      if(!$this->dbobject = @mysql_connect($this->host.":".$this->port, $this->user, $this->pass, true)){
        $this->error_code = '08001';		//SQL-client unable to establish SQL-connection
        $this->error_info = array('08001', -1, 'DB refused connection');
        return false;
      }
  
      if (!@mysql_select_db($this->database, $this->dbobject)) {
        $err = mysql_errno($this->dbobject);
        $this->error_code = ($this->err2state[$err] != '') ? $this->err2state[$err] : 'HY0000';
        $this->error_info = array($this->error_code, $err, mysql_error($this->dbobject));
        return false;
      } else {
        return true;
      }
    }
  }
  
  /**
   * Internal function to execute a query and set error code if necessary
   * @internal
   * @return bool True if succeeded
   */           
  function _MysqlQuery(){
    if (isset($this->sqlobject)) {
      return true;
    } else {
      if (($this->sqlobject = mysql_query($this->sql, $this->dbobject))===false) {
        $err = mysql_errno($this->dbobject);
        $this->error_code = ($this->err2state[$err] != '') ? $this->err2state[$err] : 'HY0000';
        $this->error_info = array($this->error_code, $err, mysql_error($this->dbobject));
        return false;
      } else {
        return true;
      }
    }
  }

  function _lastInsertId() {
    //tries to fetch last insert id and returns it
    if (!isset($this->dbobject)) {
      if(!$this->_GetConnection()){
        return false;
      }
    }
    if (!$this->sql) {
      return false;
    }
    return mysql_insert_id($this->dbobject);
  }


  var $err2state = array(
    1000 => 'HY000',
    1001 => 'HY000',
    1002 => 'HY000',
    1003 => 'HY000',
    1004 => 'HY000',
    1005 => 'HY000',
    1006 => 'HY000',
    1007 => 'HY000',
    1008 => 'HY000',
    1009 => 'HY000',
    1010 => 'HY000',
    1011 => 'HY000',
    1012 => 'HY000',
    1013 => 'HY000',
    1014 => 'HY000',
    1015 => 'HY000',
    1016 => 'HY000',
    1017 => 'HY000',
    1018 => 'HY000',
    1019 => 'HY000',
    1020 => 'HY000',
    1021 => 'HY000',
    1022 => '23000',
    1023 => 'HY000',
    1024 => 'HY000',
    1025 => 'HY000',
    1026 => 'HY000',
    1027 => 'HY000',
    1028 => 'HY000',
    1029 => 'HY000',
    1030 => 'HY000',
    1031 => 'HY000',
    1032 => 'HY000',
    1033 => 'HY000',
    1034 => 'HY000',
    1035 => 'HY000',
    1036 => 'HY000',
    1037 => 'HY001',
    1038 => 'HY001',
    1039 => 'HY000',
    1040 => '08004',
    1041 => 'HY000',
    1042 => '08S01',
    1043 => '08S01',
    1044 => '42000',
    1045 => '28000',
    1046 => '3D000',
    1047 => '08S01',
    1048 => '23000',
    1049 => '42000',
    1050 => '42S01',
    1051 => '42S02',
    1052 => '23000',
    1053 => '08S01',
    1054 => '42S22',
    1055 => '42000',
    1056 => '42000',
    1057 => '42000',
    1058 => '21S01',
    1059 => '42000',
    1060 => '42S01',
    1061 => '42000',
    1062 => '23000',
    1063 => '42000',
    1064 => 'HY000',
    1065 => '42000',
    1066 => '42000',
    1067 => '42000',
    1068 => '42000',
    1069 => '42000',
    1070 => '42000',
    1071 => '42000',
    1072 => '42000',
    1073 => '42000',
    1074 => '42000',
    1075 => '42000',
    1076 => 'HY000',
    1077 => 'HY000',
    1078 => 'HY000',
    1079 => 'HY000',
    1080 => '08S01',
    1081 => '08S01',
    1082 => '42S12',
    1083 => '42000',
    1084 => '42000',
    1085 => 'HY000',
    1086 => 'HY000',
    1087 => 'HY000',
    1088 => 'HY000',
    1089 => 'HY000',
    1090 => '42000',
    1091 => '42000',
    1092 => 'HY000',
    1093 => 'HY000',
    1094 => 'HY000',
    1095 => 'HY000',
    1096 => 'HY000',
    1097 => 'HY000',
    1098 => 'HY000',
    1099 => 'HY000',
    1100 => 'HY000',
    1101 => '42000',
    1102 => '42000',
    1103 => '42000',
    1104 => '42000',
    1105 => 'HY000',
    1106 => '42000',
    1107 => '42000',
    1108 => 'hy000',
    1109 => '42S02',
    1110 => '42000',
    1111 => 'HY000',
    1112 => '42000',
    1113 => '42000',
    1114 => 'HY000',
    1115 => '42000',
    1116 => 'HY000',
    1117 => 'HY000',
    1118 => '42000',
    1119 => 'HY000',
    1120 => '42000',
    1121 => '42000',
    1122 => 'HY000',
    1123 => 'HY000',
    1124 => 'HY000',
    1125 => 'HY000',
    1126 => 'HY000',
    1127 => 'HY000',
    1128 => 'HY000',
    1129 => 'HY000',
    1130 => 'HY000',
    1131 => '42000',
    1132 => '42000',
    1133 => '42000',
    1134 => 'HY000',
    1135 => 'HY000',
    1136 => '21S01',
    1137 => 'HY000',
    1138 => '42000',
    1139 => '42000',
    1140 => '42000',
    1141 => '42000',
    1142 => '42000',
    1143 => '42000',
    1144 => '42000',
    1145 => '42000',
    1146 => '42S02',
    1147 => '42000',
    1148 => '42000',
    1149 => '42000',
    1150 => 'HY000',
    1151 => 'HY000',
    1152 => '08S01',
    1153 => '08S01',
    1154 => '08S01',
    1155 => '08S01',
    1156 => '08S01',
    1157 => '08S01',
    1158 => '08S01',
    1159 => '08S01',
    1160 => '08S01',
    1161 => '08S01',
    1162 => '42000',
    1163 => '42000',
    1164 => '42000',
    1165 => 'HY000',
    1166 => '42000',
    1167 => '42000',
    1168 => 'HY000',
    1169 => '23000',
    1170 => '42000',
    1171 => '42000',
    1172 => '42000',
    1173 => '42000',
    1174 => 'HY000',
    1175 => 'HY000',
    1176 => 'HY000',
    1177 => '42000',
    1178 => '42000',
    1179 => '25000',
    1180 => 'HY000',
    1181 => 'HY000',
    1182 => 'HY000',
    1183 => 'HY000',
    1184 => '08S01',
    1185 => 'HY000',
    1186 => 'HY000',
    1187 => 'HY000',
    1188 => 'HY000',
    1189 => '08S01',
    1190 => '08S01',
    1191 => 'HY000',
    1192 => 'HY000',
    1193 => 'HY000',
    1194 => 'HY000',
    1195 => 'HY000',
    1196 => 'HY000',
    1197 => 'HY000',
    1198 => 'HY000',
    1199 => 'HY000',
    1200 => 'HY000',
    1201 => 'HY000',
    1202 => 'HY000',
    1203 => '42000',
    1204 => 'HY000',
    1205 => 'HY000',
    1206 => 'HY000',
    1207 => '25000',
    1208 => 'HY000',
    1209 => 'HY000',
    1210 => 'HY000',
    1211 => '42000',
    1212 => 'HY000',
    1213 => '40001',
    1214 => 'HY000',
    1215 => 'HY000',
    1216 => '23000',
    1217 => '23000',
    1218 => '08S01',
    1219 => 'HY000',
    1220 => 'HY000',
    1221 => 'HY000',
    1222 => '21000',
    1223 => 'HY000',
    1224 => 'HY000',
    1225 => 'HY000',
    1226 => '42000',
    1227 => 'HY000',
    1228 => 'HY000',
    1229 => 'HY000',
    1230 => '42000',
    1231 => '42000',
    1232 => '42000',
    1233 => 'HY000',
    1234 => '42000',
    1235 => '42000',
    1236 => 'HY000',
    1237 => 'HY000',
    1238 => 'HY000',
    1239 => '42000',
    1240 => 'HY000',
    1241 => '21000',
    1242 => '21000',
    1243 => 'HY000',
    1244 => 'HY000',
    1245 => 'HY000',
    1246 => 'HY000',
    1247 => '42S22',
    1248 => '42000',
    1249 => '01000',
    1250 => '42000',
    1251 => '08004',
    1252 => '42000',
    1253 => '42000',
    1254 => 'HY000',
    1255 => 'HY000',
    1256 => 'HY000',
    1257 => 'HY000',
    1258 => 'HY000',
    1259 => 'HY000',
    1260 => 'HY000',
    1261 => '01000',
    1262 => '01000',
    1263 => '01000',
    1264 => '01000',
    1265 => '01000',
    1266 => 'HY000',
    1267 => 'HY000',
    1268 => 'HY000',
    1269 => 'HY000',
    1270 => 'HY000',
    1271 => 'HY000',
    1272 => 'HY000',
    1273 => 'HY000',
    1274 => 'HY000',
    1275 => 'HY000',
    1276 => 'HY000',
    1277 => 'HY000',
    1278 => 'HY000',
    1279 => 'HY000',
    1280 => '42000',
    1281 => '42000',
    1282 => 'HY000',
    1283 => 'HY000',
    1284 => 'HY000',
    1285 => 'HY000',
    1286 => '42000',
    1287 => 'HY000',
    1288 => 'HY000',
    1289 => 'HY000',
    1290 => 'HY000',
    1291 => 'HY000',
    1292 => 'HY000',
    1293 => 'HY000',
    1294 => 'HY000',
    1295 => 'HY000',
    1296 => 'HY000',
    1297 => 'HY000',
    1298 => 'HY000',
    1299 => 'HY000',
    1300 => 'HY000',
    1301 => 'HY000',
    1302 => 'HY000',
  );
}

/**** Listing of SQL States

         _Condition__________________Class_Subcondition_______________Subclass

        | ambiguous cursor name    | 3C  | (no subclass)            | 000  |
        |                          |     |                          |      |
        | cardinality violation    | 21  | (no subclass)            | 000  |
        |                          |     |                          |      |
        | connection exception     | 08  | (no subclass)            | 000  |
        |                          |     |                          |      |
        |                          |     | connection does not      | 003  |
        |                          |     | exist                    |      |
        |                          |     |                          |      |
        |                          |     | connection failure       | 006  |
        |                          |     |                          |      |
        |                          |     | connection name in use   | 002  |
        |                          |     |                          |      |
        |                          |     | SQL-client unable to     | 001  |
                                           establish SQL-connection
        |                          |     |                          |      |
        |                          |     | SQL-server rejected      | 004  |
        |                          |     | establishment of SQL-    |      |
                                           connection
        |                          |     |                          |      |
        |                          |     | transaction resolution   | 007  |
        |                          |     | unknown                  |      |
        |                          |     |                          |      |
        | data exception           | 22  | (no subclass)            | 000  |
        |                          |     |                          |      |
        |                          |     | character not in reper-  | 021  |
                                           toire
        |                          |     |                          |      |
        |                          |     | datetime field overflow  | 008  |
        |                          |     |                          |      |
        |                          |     | division by zero         | 012  |
        |                          |     |                          |      |
        |                          |     | error in assignment      | 005  |
        |                          |     |                          |      |
        |                          |     | indicator overflow       | 022  |
        |                          |     |                          |      |
        |                          |     | interval field overflow  | 015  |
        |                          |     |                          |      |
        |                          |     | invalid character value  | 018  |
                                           for cast
        |                          |     |                          |      |
        |                          |     | invalid datetime format  | 007  |
        |                          |     |                          |      |
        |                          |     | invalid escape character | 019  |
        |                          |     |                          |      |
        |                          |     | invalid escape sequence  | 025  |
        |                          |     |                          |      |
        |                          |     | invalid parameter value  | 023  |
        |                          |     |                          |      |
        |                          |     | invalid time zone dis-   | 009  |
                                           placement value
        |                          |     |                          |      |
        |                          |     | null value, no indicator | 002  |
        |                          |     | parameter                |      |
        |                          |     |                          |      |
        |                          |     | numeric value out of     | 003  |
        |                          |     | range                    |      |
        |                          |     |                          |      |
        |                          |     | string data, length      | 026  |
        |                          |     | mismatch                 |      |
        |                          |     |                          |      |
        |                          |     | string data, right       | 001  |
                                           truncation
        |                          |     |                          |      |
        |                          |     | substring error          | 011  |
        |                          |     |                          |      |
        |                          |     | trim error               | 027  |
        |                          |     |                          |      |
        |                          |     | unterminated C string    | 024  |
        |                          |     |                          |      |
        | dependent privilege      | 2B  | (no subclass)            | 000  |
          descriptors still exist

        | dynamic SQL error        | 07  | (no subclass)            | 000  |
        |                          |     |                          |      |
        |                          |     | cursor specification     | 003  |
                                           cannot be executed
        |                          |     |                          |      |
        |                          |     | invalid descriptor count | 008  |
        |                          |     |                          |      |
        |                          |     | invalid descriptor index | 009  |
        |                          |     |                          |      |
        |                          |     | prepared statement not a | 005  |
                                           cursor specification
        |                          |     |                          |      |
        |                          |     | restricted data type     | 006  |
        |                          |     | attribute violation      |      |
        |                          |     |                          |      |
        |                          |     | using clause does not    | 001  |
        |                          |     | match dynamic parameter  |      |
                                           specifications
        |                          |     |                          |      |
        |                          |     | using clause does not    | 002  |
        |                          |     | match target specifica-  |      |
        |                          |     | tions                    |      |
        |                          |     |                          |      |
        |                          |     | using clause required    | 004  |
        |                          |     | for dynamic parameters   |      |
        |                          |     |                          |      |
        |                          |     | using clause required    | 007  |
        |                          |     | for result fields        |      |
        |                          |     |                          |      |
        | feature not supported    | 0A  | (no subclass)            | 000  |
        |                          |     |                          |      |
        |                          |     | multiple server transac- | 001  |
                                           tions
        |                          |     |                          |      |
        | integrity constraint     | 23  | (no subclass)            | 000  |
        | violation                |     |                          |      |
        |                          |     |                          |      |
        | invalid authorization    | 28  | (no subclass)            | 000  |
        | specification            |     |                          |      |
        |                          |     |                          |      |
        | invalid catalog name     | 3D  | (no subclass)            | 000  |
        |                          |     |                          |      |
        | invalid character set    | 2C  | (no subclass)            | 000  |
          name
        |                          |     |                          |      |
        | invalid condition number | 35  | (no subclass)            | 000  |
        |                          |     |                          |      |
        | invalid connection name  | 2E  | (no subclass)            | 000  |
        |                          |     |                          |      |
        | invalid cursor name      | 34  | (no subclass)            | 000  |
        |                          |     |                          |      |
        | invalid cursor state     | 24  | (no subclass)            | 000  |
        |                          |     |                          |      |
        | invalid schema name      | 3F  | (no subclass)            | 000  |
        |                          |     |                          |      |
        | invalid SQL descriptor   | 33  | (no subclass)            | 000  |
          name
        |                          |     |                          |      |
        | invalid SQL statement    | 26  | (no subclass)            | 000  |
        | name                     |     |                          |      |
        |                          |     |                          |      |
        | invalid transaction      | 25  | (no subclass)            | 000  |
        | state                    |     |                          |      |
        |                          |     |                          |      |
        | invalid transaction      | 2D  | (no subclass)            | 000  |
        | termination              |     |                          |      |
        |                          |     |                          |      |
        | no data                  | 02  | (no subclass)            | 000  |
        |                          |     |                          |      |
        | Remote Database Access   | HZ  | (See ISO/IEC 9579-2 for  |      |
                                           the definition of pro-
                                           tocol subconditions and
                                           subclass code values)
        |                          |     |                          |      |
        | successful completion    | 00  | (no subclass)            | 000  |
        |                          |     |                          |      |
        | syntax error or access   | 42  | (no subclass)            | 000  |
        | rule violation           |     |                          |      |
        |                          |     |                          |      |
        | syntax error or access   | 2A  | (no subclass)            | 000  |
        | rule violation in direct |     |                          |      |
          SQL statement
        |                          |     |                          |      |
        | syntax error or ac-      | 37  | (no subclass)            | 000  |
        | cess rule violation in   |     |                          |      |
        | dynamic SQL statement    |     |                          |      |
        |                          |     |                          |      |
        | transaction rollback     | 40  | (no subclass)            | 000  |
        |                          |     |                          |      |
        |                          |     | integrity constraint     | 002  |
        |                          |     | violation                |      |

        |                          |     | serialization failure    | 001  |
        |                          |     |                          |      |
        |                          |     | statement completion     | 003  |
                                           unknown

        | triggered data change    | 27  | (no subclass)            | 000  |
        | violation                |     |                          |      |
        |                          |     |                          |      |
        | warning                  | 01  | (no subclass)            | 000  |
        |                          |     |                          |      |
        |                          |     | cursor operation con-    | 001  |
                                           flict

        |                          |     | disconnect error         | 002  |
        |                          |     |                          |      |
        |                          |     | implicit zero-bit        | 008  |
                                           padding

        |                          |     | insufficient item de-    | 005  |
        |                          |     | scriptor areas           |      |
        |                          |     |                          |      |
        |                          |     | null value eliminated in | 003  |
        |                          |     | set function             |      |
        |                          |     |                          |      |
        |                          |     | privilege not granted    | 007  |
        |                          |     |                          |      |
        |                          |     | privilege not revoked    | 006  |
        |                          |     |                          |      |
        |                          |     | query expression too     | 00A  |
                                           long for information
                                           schema

        |                          |     | search condition too     | 009  |
        |                          |     | long for information     |      |
        |                          |     | schema                   |      |
        |                          |     |                          |      |
        |                          |     | string data, right       | 004  |
        |                          |     | truncation               |      |
        |                          |     |                          |      |
        | with check option viola- | 44  | (no subclass)            | 000  |
        |_tion_____________________|_____|__________________________|______|
        |                          |     |                          |      |
        |                          |     |                          |      |
        |                          |     |                          |      |

**************************/
?>